@@ -2,7 +2,7 @@ package MVC::Neaf::Util::Container;
 
 use strict;
 use warnings;
-our $VERSION = 0.2603;
+our $VERSION = '0.27';
 
 =head1 NAME
 
@@ -217,7 +217,8 @@ Same as fetch(), but only return the last (last added & longest path) element.
 
 sub fetch_last {
     my $self = shift;
-    return [ $self->fetch_raw(@_) ]->[-1]{data};
+    my ($bucket) = reverse $self->fetch_raw(@_);
+    return $bucket->{data};
 };
 
 =head2 fetch_raw
@@ -261,7 +262,7 @@ sub fetch_raw {
 
 This module is part of L<MVC::Neaf> suite.
 
-Copyright 2016-2018 Konstantin S. Uvarin C<khedin@cpan.org>.
+Copyright 2016-2019 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
